game = {
	level = 1,
	object_tick = 0,
	object_types = {},
	progress_bar = {
		value = 0,
		value_x = 0,
		value_width = 0,
		panel_color = {0, 0, 150, 255},
		panel_outline_color = {0, 0, 200, 255},
		bar_color = {0, 150, 128, 255},
		bar_background_color = {0, 0, 64, 255},
		bar_outline_color = {0, 0, 200, 255},
		panel_x = 1,
		panel_y = 1,
		panel_width = screen.width - 2,
		panel_height = screen.height / 10
	}
}


function game:reset()
	platform:reset()
	if self.won then self.level = self.level + 1 end
	self.objects = {}
	self.pause = false
	self.won = false
	self.lost = false
	self.target_size = (screen.width / 8) + (self.level * 4)
	self.object_spawnRate = 2 + (self.level * 1.1)
	self.initial_objects = self.object_spawnRate * 5

	--Types setup
	self.object_type = {
			{name = "scale_up", width = screen.width / 40, height = screen.height / 40, color = {60, 255, 128, 255}, fall_speed = (screen.height / 12) + (self.level * 2), pickup = scale_up},
			{name = "scale_down", width = screen.width / 40, height = screen.height / 40, color = {18, 200, 128, 255}, fall_speed = (screen.height / 12) + (self.level * 2), pickup = scale_down}
		}

	for i=1, self.initial_objects do
		local number = love.math.random()
		local t = 1
		if number < 0.5 then
			t = 2
		end

		local x = math.random(0, screen.width - self.object_type[t].width)
		local y = math.random(0, screen.height - (self.object_type[t].height * 16))
		self:spawnObj(t, x, y)
	end
	--Progress bar
	self.progress_bar.width = self.progress_bar.panel_width / 1.1
	self.progress_bar.height = self.progress_bar.panel_height / 3
	self.progress_bar.x = (self.progress_bar.panel_width / 2) - (self.progress_bar.width / 2)
	self.progress_bar.y = (self.progress_bar.panel_height / 2) - (self.progress_bar.height / 2)
end


function game:update(dt)
	platform:update(dt)

	if not self.pause and not self.lost and not self.won  then
		--Spawning fuckers
		self.object_tick = self.object_tick + dt
		if self.object_tick > (1 / self.object_spawnRate) then
			local number = love.math.random()
			local t = 1
			if number < 0.5 then
				t = 2
			end

			local x = math.random(0, screen.width - self.object_type[t].width)
			local y = -self.object_type[t].height
			self:spawnObj(t, x, y)

			self.object_tick = 0 
		end
	end
	for i,v in ipairs(self.objects) do
		v:update(dt)

		--COLLISION CHECK
		if platform:collision(v) then
			if not v.remove then
				v.remove = true
				self.object_type[v.type].pickup()
				if platform.width == platform.default_width and platform.impending_loss > 1 then
					platform:setSize(platform.default_width, platform.default_height)
					self.lost = true
				elseif platform.width == self.target_size then
					platform:setSize(self.target_size, self.target_size)
					self.won = true
				end
			end
		end
		--Bottom Collision
		if (v.y + v.height) > screen.height then
			if not v.remove then
				v.remove = true
			end
		end
		--REMOVING DEAD FUCKERS
		if v.removed and not self.pause then
			table.remove(self.objects, i)
		end
	end
	--PROGRESS_BAR
	self.progress_bar.value = (self.progress_bar.width / (self.target_size - platform.default_width)) * (platform.width - platform.default_width)
	if self.won or self.lost then
		self.progress_bar.value = 0
	end
	self.progress_bar.value_width = self.progress_bar.value_width + (self.progress_bar.value - self.progress_bar.value_width) * 0.1
	self.progress_bar.value_x = ((self.progress_bar.width / 2) - (self.progress_bar.value_width / 2)) + self.progress_bar.x
end

function game:draw()
	platform:draw()
	for i,v in ipairs(self.objects) do
		v:draw()
	end

	--Background Darkening
	if self.lost or self.pause then
		love.graphics.setColor(hsl(0, 0, 0, 128))
		love.graphics.rectangle("fill", 0, 0, screen.width, screen.height)
	elseif self.won then
		love.graphics.setColor(hsl(0, 0, 255, 110))
		love.graphics.rectangle("fill", 0, 0, screen.width, screen.height)
	end

	--Lost won pause
	love.graphics.setFont(font)
	if self.lost then
		love.graphics.setColor(hsl(0, 0, 255, 255))
		love.graphics.printf("You lost", 0, screen.width / 2, screen.width, "center")
	elseif self.won then
		love.graphics.setColor(hsl(0, 0, 255, 255))
		love.graphics.printf("You won", 0, screen.width / 2, screen.width, "center")
	elseif self.pause then
		love.graphics.setColor(hsl(0, 0, 255, 255))
		love.graphics.printf("Shits paused.", 0, screen.width / 2, screen.width, "center")
	end

	--PROGRESS BAR

	--panel_bg
	love.graphics.setColor(hsl(self.progress_bar.panel_color))
	love.graphics.rectangle("fill", self.progress_bar.panel_x, self.progress_bar.panel_y, self.progress_bar.panel_width, self.progress_bar.panel_height)
	--panel bg outline
	love.graphics.setColor(hsl(self.progress_bar.panel_outline_color))
	love.graphics.rectangle("line", self.progress_bar.panel_x, self.progress_bar.panel_y, self.progress_bar.panel_width, self.progress_bar.panel_height)
	--bar bg
	love.graphics.setColor(hsl(self.progress_bar.bar_background_color))
	love.graphics.rectangle("fill", self.progress_bar.x, self.progress_bar.y, self.progress_bar.width, self.progress_bar.height)
	--bar value
	love.graphics.setColor(hsl(60 * ((1 / self.progress_bar.width) * self.progress_bar.value), self.progress_bar.bar_color[2], self.progress_bar.bar_color[3], self.progress_bar.bar_color[4]))
	love.graphics.rectangle("fill", self.progress_bar.value_x, self.progress_bar.y, self.progress_bar.value_width, self.progress_bar.height)

	--Bar outline
	love.graphics.setColor(hsl(self.progress_bar.bar_outline_color))
	love.graphics.rectangle("line", self.progress_bar.x, self.progress_bar.y, self.progress_bar.width, self.progress_bar.height)

	love.graphics.setFont(font2)
	love.graphics.setColor(0, 0, 0, 255)
	love.graphics.printf("Level: "..self.level, 0, (self.progress_bar.panel_y + self.progress_bar.panel_height) - love.graphics.getFont():getHeight() - 3, screen.width, "center")
end

function game:keypressed(key)
	if key == " " then
		if self.lost or self.won then
			self:reset()
		end
	end
end

function  game:mousepressed(x, y, key)
	if key == "l" then
	
	end
end

function game:spawnObj(type, x, y, width, height, color, fall_speed)
	type = type or 1
	local t = self.object_type[type]
	self.objects[#self.objects + 1] = obj.new(type, x, y, t.width, t.height, {t.color[1], t.color[2], t.color[3], t.color[4]}, t.fall_speed + math.random(-8, 8))
end

function game:getGravity()
	return self.gravity
end

function game:getMaxVelocity()
	return self.max_velocity
end

--OBJECT PICKUP FUNCTIONS

function scale_up()
	platform:addSize(screen.width / 96, screen.width / 96)
	platform.impending_loss = 0
end

function scale_down()
	platform:setSize(platform.default_width, platform.default_height)
	platform.impending_loss = platform.impending_loss + 1
end