local WINDOW_TITLE = "Untitled"

screen = {
	width = love.window.getWidth(),
	height = love.window.getHeight()
}

--Loading Assets
font = love.graphics.newFont(screen.width / 24)
font2 = love.graphics.newFont(screen.width / 50)

--Loading Classes
require "platform"
require "obj"
require "game"


--Game state
local state = {ingame = "ingame", menu = "menu"}
local game_state = state.ingame

--Debug
local debug_enabled = false

function love.load()
	love.graphics.setLineWidth(1)
	love.graphics.setLineStyle("rough")
	if game_state == state.ingame then
		game:reset()
	end
	love.graphics.setBackgroundColor(hsl(0, 0, 64))
end

function love.update(dt)
	--love.window.setTitle(WINDOW_TITLE.." | FPS:"..love.timer.getFPS())
	if game_state == state.ingame then
		game:update(dt)
	end
end

function love.draw()
	if game_state == state.ingame then
		game:draw()
	end
	if debug_enabled then
		love.graphics.setColor(0, 0, 0, 128)
		love.graphics.rectangle("fill", 0, 0, screen.height, 12 * 3)

		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.setBlendMode("alpha")
		love.graphics.print("game_state: "..tostring(game_state)..
		"\nObjects: "..#game.objects..
		"\nplatform["..platform.width.."]["..platform.height.."]"..
		"\nImpending_loss = "..platform.impending_loss, 12, 12)

		love.graphics.rectangle("line", platform.x, platform.y, platform.width, platform.height)
	end
end

function love.keypressed(key)
	if key == "escape" then love.event.push("quit") end
	if game_state == state.ingame then
		game:keypressed(key)
	end
	if key == "tab" then debug_enabled = not debug_enabled end
end

function love.mousepressed(x, y, key)
	if game_state == state.ingame then
		game:mousepressed(x, y, key)
	end
end

function collision(x1,y1,w1,h1, x2,y2,w2,h2)
  return x1 < x2+w2 and
         x2 < x1+w1 and
         y1 < y2+h2 and
         y2 < y1+h1
end

function hsl(h, s, l, a)
	if type(h) == "table" then
		s = h[2]
		l = h[3]
		a = h[4]
		h = h[1]
	end

    if s<=0 then return l,l,l,a end
    h, s, l = h/256*6, s/255, l/255
    local c = (1-math.abs(2*l-1))*s
    local x = (1-math.abs(h%2-1))*c
    local m,r,g,b = (l-.5*c), 0,0,0
    if h < 1     then r,g,b = c,x,0
    elseif h < 2 then r,g,b = x,c,0
    elseif h < 3 then r,g,b = 0,c,x
    elseif h < 4 then r,g,b = 0,x,c
    elseif h < 5 then r,g,b = x,0,c
    else              r,g,b = c,0,x
    end return {(r+m)*255,(g+m)*255,(b+m)*255,a}
end