platform = {
	default_width = screen.width / 24,
	default_height = screen.height / 24,
	bottom_offset = 12,
	color = {255, 255, 255, 255},
	smooth_multiplier = 0.1,
	movement_speed = screen.width / 1.5,
	alpha = 1
}

function platform:reset()
	self.width = self.default_width
	self.draw_width = self.width
	self.height = self.default_height
	self.draw_height = self.height
	self.x = (screen.width / 2) - (self.width / 2)
	self.y = screen.height - self.height - self.bottom_offset
	self.draw_x = self.x
	self.draw_y = self.y + self.height + self.bottom_offset
	self.impending_loss = 0
	--self.alpha = 1
end

function platform:update(dt)
	if not game.lost and not game.pause and not game.won then
		if love.keyboard.isDown("w") or love.keyboard.isDown("up") then
			self.y = self.y - self.movement_speed * dt
		elseif love.keyboard.isDown("s") or love.keyboard.isDown("down") then
			self.y = self.y + self.movement_speed * dt
		end
		if love.keyboard.isDown("a") or love.keyboard.isDown("left") then
			self.x = self.x - self.movement_speed * dt
		elseif love.keyboard.isDown("d") or love.keyboard.isDown("right") then
			self.x = self.x + self.movement_speed * dt
		end
	end

	if self.draw_x < (self.width / 2) then 
		self.x = (self.width / 2) 
		self.draw_x = (self.width / 2) 
	elseif (self.draw_x + (self.width / 2)) > screen.width then 
		self.x  = screen.width - (self.width / 2) 
		self.draw_x  = screen.width - (self.width / 2) 
	end

	if self.draw_y < (self.height / 2) + (game.progress_bar.panel_y + game.progress_bar.panel_height) then 
		self.y = (self.height / 2) + (game.progress_bar.panel_y + game.progress_bar.panel_height) 
		self.draw_y = (self.height / 2) + (game.progress_bar.panel_y + game.progress_bar.panel_height) 
	elseif (self.draw_y + (self.height / 2)) > screen.height then 
		self.y = screen.height - (self.height / 2) 
		self.draw_y = screen.height - (self.height / 2) 
	end
	
	self.draw_x = self.draw_x + (self.x - self.draw_x) * self.smooth_multiplier
	self.draw_y = self.draw_y + (self.y - self.draw_y) * self.smooth_multiplier
	self.draw_width = self.draw_width + (self.width - self.draw_width) * self.smooth_multiplier
	self.draw_height = self.draw_height + (self.height - self.draw_height) * self.smooth_multiplier

	if game.lost or game.won then
		self.alpha = self.alpha + (0 - self.alpha) * 0.02
		self.width = 0
		self.height = 0
	else
		self.alpha = self.alpha + (1 - self.alpha) * 0.1
	end
end

function platform:draw()
	if self.impending_loss > 0 then
		self.color[1] = 0
		self.color[3] = 126
	else
		self.color[1] = 255
		self.color[3] = 255
	end 
	love.graphics.setColor(hsl(self.color[1], self.color[2], self.color[3], (self.color[4] / 8) * self.alpha))
	love.graphics.rectangle("fill", self.draw_x - ((self.draw_width / 2) * self.alpha), self.draw_y - ((self.draw_height / 2) * self.alpha), self.draw_width * self.alpha, self.draw_height * self.alpha)

	love.graphics.setColor(hsl(self.color[1], self.color[2], self.color[3], self.color[4] * self.alpha))
	love.graphics.rectangle("line", self.draw_x - ((self.draw_width / 2) * self.alpha), self.draw_y - ((self.draw_height / 2) * self.alpha), self.draw_width * self.alpha, self.draw_height * self.alpha)
end

function platform:addSize(width, height)
	self.width = self.width + width
	if self.width < 0 then self.width = 0 end
	if self.width > game.target_size then self.width = game.target_size end
	self.height = self.height + height
	if self.height < 0 then self.height = 0 end
	if self.height > game.target_size then self.height = game.target_size end
end

function platform:setSize(width, height)
	self.width = width
	self.height = height
end

function platform:collision(obj)
	if collision(self.draw_x - (self.draw_width / 2), self.draw_y - (self.draw_height / 2), self.draw_width, self.draw_height, obj.x, obj.y, obj.width, obj.height) then
		return true
	else
		return false
	end
end